from glob import glob
from .constants import BRIDGES, ZURICH
from os.path import join, dirname, isfile, basename
from os import makedirs
from shutil import copyfile
from .utils import parse_sl_table
from .sftp_server import SFTPKnex
from sys import stderr


def hauid_from_metaspades_dir(contig_path, sl_tbl):
    tkns = contig_path.split('haib')[1].split('/')
    haib = 'haib' + tkns[0]
    flowcell = tkns[1]
    raw_id = tkns[2].split('_1.fastq.gz.metaspades')[0].split('.R1.fastq.gz.metaspades')[0]
    sl_id = sl_tbl[raw_id].split('_')[2]
    return haib, flowcell, sl_id


def get_bridges_metaspades_dirs():
    contig_files = glob(BRIDGES.METASUB_DATA + '/**/contigs.fasta', recursive=True)
    metaspades_dirs = {
        dirname(contig_file)
        for contig_file in contig_files
    }
    return metaspades_dirs


def upload_one_metaspades_dir(server, mspades_dir, sl_tbl):
    hauid_tuple = hauid_from_metaspades_dir(mspades_dir, sl_tbl)
    hauid_sname = '_'.join(hauid_tuple)
    remote_dir = f'{ZURICH.ASSEMBLIES}/{hauid_tuple[0]}/{hauid_tuple[1]}/{hauid_sname}.metaspades/'
    server.make_dirs(remote_dir)
    for subfile in glob(mspades_dir + '/*'):
        if not isfile(subfile):
            continue
        remote_path = remote_dir + basename(subfile)
        server.upload_file(subfile, remote_path)
    misc_dir = remote_dir + 'misc/'
    server.make_dirs(misc_dir)
    for misc_file in glob(mspades_dir + 'misc/*'):
        if not isfile(subfile):
            continue
        remote_path = misc_dir + basename(misc_file)
        server.upload_file(misc_file, remote_path)


def copy_one_metaspades_dir(target_dir, mspades_dir, sl_tbl):
    hauid_tuple = hauid_from_metaspades_dir(mspades_dir, sl_tbl)
    hauid_sname = '_'.join(hauid_tuple)
    target_dir_path = f'{target_dir}/{hauid_tuple[0]}/{hauid_tuple[1]}/{hauid_sname}.metaspades/'
    makedirs(target_dir_path, exist_ok=True)
    for subfile in glob(mspades_dir + '/*'):
        if not isfile(subfile):
            continue
        target_file_path = target_dir_path + basename(subfile)
        if not isfile(target_file_path):
            copyfile(subfile, target_file_path)
    misc_dir = target_dir_path + 'misc/'
    makedirs(misc_dir, exist_ok=True)
    for misc_file in glob(mspades_dir + 'misc/*'):
        if not isfile(subfile):
            continue
        target_file_path = misc_dir + basename(misc_file)
        if not isfile(target_file_path):
            copyfile(misc_file, target_file_path)


def upload_metaspades_assemblies_from_bridges(username, password, dryrun=False):
    sl_tbl = parse_sl_table()
    try:
        server = SFTPKnex(username, password, dryrun=dryrun)
        for metaspades_dir in get_bridges_metaspades_dirs():
            try:
                upload_one_metaspades_dir(server, metaspades_dir, sl_tbl)
            except IndexError:
                print(f'NO_UPLOAD INDEX_ERROR {metaspades_dir}', file=stderr)
            except KeyError:
                print(f'NO_UPLOAD KEY_ERROR {metaspades_dir}', file=stderr)
    finally:
        server.close()


def copy_metaspades_assemblies_from_bridges(target_dir):
    sl_tbl = parse_sl_table()
    for metaspades_dir in get_bridges_metaspades_dirs():
        try:
            copy_one_metaspades_dir(target_dir, metaspades_dir, sl_tbl)
        except IndexError:
            print(f'NO_COPY INDEX_ERROR {metaspades_dir}', file=stderr)
        except KeyError:
            print(f'NO_COPY KEY_ERROR {metaspades_dir}', file=stderr)



"""
Procedure for handling data from HA:
1. Each new flowcell url should be recorded in 'hudson_alpha_flowcells.csv'
   along with its metasub_project_name, and H.A. project names
2. A new 'library' directory should be created as <library_top>/<HA project ID>/<flowcell number>
3. The relevant file and filenames files hould be downloaded to this dir
4. All fragmented files should be downloaded to the new dir
5. All fragmented files should be concatenated within the dir and the fragments deleted.
   Each file should now be given a name of the form 
   <HA project ID>_<flowcell number>_<SL number>_[1,2].fastq.gz
   The predicate of this name is the HA Unique ID

--- 
Procedure for back mapping results to HALib
1. Make a map from current name to the HA Unique ID
2. Build a table mapping old files to new names
3. move and link back old files to new names
"""

from .constants import HALPHA, ATHENA
from glob import glob
from os import makedirs
from os.path import join, basename, dirname, isfile
from concurrent.futures import ThreadPoolExecutor
import requests
from sys import stderr


###############################################################################
"""
Make migrations from current (aug 22, 2018) state of Athena to new HA Library based state.
"""


def rename_one_sl_path_to_ha_unique(filepath):
    tkns = filepath.split('/')
    ha_project_id, flowcell_number, file_base = tkns[-3], tkns[-2], tkns[-1]
    new_file_base = f'{ha_project_id}_{flowcell_number}_{file_base}'
    new_filepath = join(dirname(filepath), new_file_base)
    return new_filepath


def rename_sl_names_to_ha_unique():
    for filepath in glob(HALPHA.ATHENA_SL_LIBRARY + '/*/*/SL*.fastq.gz'):
        new_filepath = rename_one_sl_path_to_ha_unique(filepath)
        if not isfile(new_filepath):
            print(f'{filepath} {new_filepath}')


def map_from_name_in_datasuper_to_ha_unique(source_fastqs_file):
    name_map = {}
    for line in source_fastqs_file:
        if len(line.strip()) == 0:
            continue
        tkns = line.split()
        if '1.fastq' not in tkns[0]:
            continue
        ds_name = basename(tkns[0]).split('_1.')[0].split('.R1.')[0]
        try:
            ha_unique = basename(rename_one_sl_path_to_ha_unique(tkns[1])).split('_1.fastq.gz')[0]
        except IndexError:
            print('!!! ' + line, file=stderr)
        assert ds_name not in name_map
        name_map[ds_name] = ha_unique
    return name_map


def rename_existing_core_results(source_fastqs_file, old_result_dir):
    name_map = map_from_name_in_datasuper_to_ha_unique(source_fastqs_file)
    path_map = {}
    for filepath in glob(old_result_dir + '/*/*'):
        old_name = basename(filepath).split('.')[0]
        try:
            new_name = name_map[old_name]
        except KeyError:
            print(f'!!! {old_name}', file=stderr)
            continue
        new_path = 'new_metasub_cap'.join(filepath.split('metasub_cap'))
        new_path = new_name.join(new_path.split(old_name))
        path_map[filepath] = new_path
    return path_map


###############################################################################

"""
Download data from Hudson Alpha.
"""


def download_file(url, path, auth):
    r = requests.get(url, auth=auth)
    open(path, 'wb').write(r.content)


def process_flowcells(dryrun, ha_auth):
    with open(HALPHA.FLOWCELL_FILENAME) as flowcell_file:
        tkn_gen = (
            line.strip().split(',')
            for line in flowcell_file
            if len(line.strip()) > 0
        )
        for metasub_project_name, _, ha_project_id, ha_file_url in tkn_gen:
            flowcell_number = ha_file_url.split('files_')[-1].split('.txt')[0]
            ha_file_url = HALPHA.URL + ha_file_url
            ha_filename_url = 'filenames'.join(ha_file_url.split('files'))
            library_dir = join(HALPHA.ATHENA_SL_LIBRARY, ha_project_id, flowcell_number)
            makedirs(library_dir, exist_ok=True)

            ha_file_path = join(library_dir, ha_file_url.split('/')[-1])
            download_file(ha_file_url, ha_file_path, ha_auth)
            ha_filename_path = join(library_dir, ha_filename_url.split('/')[-1])
            download_file(ha_filename_url, ha_filename_path, ha_auth)
            handle_single_flowcell(
                dryrun, ha_auth,
                library_dir,
                metasub_project_name, ha_project_id, flowcell_number,
                ha_file_path, ha_filename_path
            )


def parse_filename(ha_filename_path):
    name_map = {}
    with open(ha_filename_path) as hfp:
        hfp.readline()
        hfp.readline()
        for line in hfp:
            line = line.strip()
            if len(line) == 0:
                continue
            tkns = line.split('\t')
            try:
                slname, trip_name, = tkns[2], tkns[3]
            except IndexError:
                continue
            if len(tkns) >= 6:
                description_name = tkns[5].split()[0]
                name_map[description_name] = slname
            name_map[slname] = slname
            name_map[trip_name] = slname
    return name_map


def get_root_and_read_number(filepath):
    filename = basename(filepath)
    if '_1.' in filename:
        return filename.split('_1.')[0], '1'
    elif '.R1.' in filename:
        return filename.split('.R1.')[0], '1'
    elif '_2.' in filename:
        return filename.split('_2.')[0], '2'
    elif '.R2.' in filename:
        return filename.split('.R2.')[0], '2'
    assert False, filepath


def download_files(dryrun, ha_auth, library_dir, ha_file_path, existing_ha_uids):
    existing_slnames = {hauid.split('_')[-1] for hauid in existing_ha_uids}
    executor = ThreadPoolExecutor(max_workers=50)
    futures = []
    with open(ha_file_path) as hfp:
        for line in hfp:
            line = line.strip()
            if len(line) == 0:
                continue
            slname = line.split('.fastq.gz')[0].split('_')[-1]
            if slname in existing_slnames:
                continue
            url = line
            path = join(library_dir, url.split('/')[-1])
            if isfile(path):
                continue
            print(f'DOWNLOAD\t{url}\t{path}')
            if not dryrun:
                futures.append(executor.submit(lambda : download_file(url, path, ha_auth)))
    for future in futures:
        future.result()


def handle_single_flowcell(dryrun, ha_auth,
                           library_dir,
                           metasub_project_name, ha_project_id, flowcell_number,
                           ha_file_path, ha_filename_path):
    names_in_library = set()
    for in_library in glob(library_dir + '/*.fastq.gz'):
        try:
            root, _ = get_root_and_read_number(in_library)
            names_in_library.add(root)
        except AssertionError:
            continue
    download_files(dryrun, ha_auth, library_dir, ha_file_path, names_in_library)



from glob import glob
from os.path import isfile
from subprocess import call
from .utils import (
    get_complete_metadata,
    get_canonical_city_names,
    as_display_name,
)
from tempfile import NamedTemporaryFile
from .constants import COLUMNS, METAGENSCOPE


def run_cmd(cmd, dryrun=True):
    print(cmd)
    if dryrun:
        return
    call(cmd, shell=True)


def get_sample_names(city_names):
    """Return a list of sample names for the city."""
    city_names = [city_name.lower() for city_name in city_names]
    metadata = get_complete_metadata()
    sample_names = set()
    for row_id, row in metadata.iterrows():
        if str(row[COLUMNS.CITY]).lower() not in city_names:
            continue
        sample_names.add(row['hudson_alpha_uid'])
    return sample_names


def build_file_manifest(result_dir, sample_names):
    """Return the name of a temp file of filepaths from the given city."""
    files = []
    sample_names_with_files = set()
    for sample_name in sample_names:
        for fpath in glob(result_dir + '/' + sample_name + '/*'):
            if isfile(fpath):
                files.append(fpath)
                sample_names_with_files.add(sample_name)
    city_metadata_file_handle = NamedTemporaryFile(mode='w', delete=False)
    for fpath in files:
        city_metadata_file_handle.write(f'{fpath}\n')
    city_metadata_file_handle.close()
    return city_metadata_file_handle.name, sample_names_with_files


def build_metadata_table(city_names):
    """Return the name of a temp file with metadata from the given city."""
    metadata = get_complete_metadata(uploadable=True)
    city_metadata_rows = metadata[COLUMNS.CITY].str.lower().isin(city_names)
    city_metadata = metadata[city_metadata_rows]
    city_metadata_file_handle = NamedTemporaryFile(suffix='.csv', mode='w', delete=False)
    city_metadata_file = city_metadata_file_handle.name
    city_metadata_file_handle.close()
    city_metadata.to_csv(city_metadata_file)

    return city_metadata_file


def upload_cities(result_dir, city_names, display_name, upload_only=False, dryrun=True):
    """Upload a city to MGS and run middleware."""
    city_names = {city_name.lower() for city_name in city_names}
    assert len(city_names & get_canonical_city_names(lower=True)) == len(city_names)

    sample_names = get_sample_names(city_names)
    file_manifest, sample_names = build_file_manifest(result_dir, sample_names)
    upload_files_cmd = f'metagenscope upload files -g "{display_name}" -m {file_manifest}'
    run_cmd(upload_files_cmd, dryrun=dryrun)

    city_metadata_table = build_metadata_table(city_names)
    sample_name_str = ' '.join(sample_names)
    upload_metadata_cmd = f'metagenscope upload metadata {city_metadata_table} {sample_name_str}'
    run_cmd(upload_metadata_cmd, dryrun=dryrun)

    add_group_cmd = f'metagenscope add group-to-org "{display_name}" {METAGENSCOPE.MSUB_GROUP_NAME}'
    run_cmd(add_group_cmd, dryrun=dryrun)

    if upload_only:
        return
    group_middleware_cmd = f'metagenscope run middleware group "{display_name}"'
    run_cmd(group_middleware_cmd, dryrun=dryrun)
    for sample_name in sample_names:
        sample_middleware_cmd = f'metagenscope run middleware samples {sample_name}'
        run_cmd(sample_middleware_cmd, dryrun=dryrun)



from argparse import Namespace
from os.path import join, dirname


CODE_DIR = dirname(dirname(__file__))


# Metadata
METADATA = Namespace()
METADATA.COMPLETE_TABLE_URL = 'https://raw.githubusercontent.com/dcdanko/MetaSUB-metadata/master/complete_metadata.csv'
METADATA.UPLOADABLE_TABLE_URL = 'https://raw.githubusercontent.com/dcdanko/MetaSUB-metadata/master/upload_metadata.csv'
METADATA.CANONICAL_CITIES_URL = 'https://raw.githubusercontent.com/dcdanko/MetaSUB-metadata/master/spreadsheets/city_names.csv'

# MetaGenScope
METAGENSCOPE = Namespace()
METAGENSCOPE.MSUB_GROUP_NAME = 'MetaSUB'

# Columns
COLUMNS = Namespace()
COLUMNS.BC = 'barcode'
COLUMNS.METASUB_NAME = 'metasub_name'
COLUMNS.HA_ID = 'ha_id'
COLUMNS.SL_NAME = 'sl_name'
COLUMNS.CITY = 'city'
COLUMNS.IDS = set(
    [COLUMNS.HA_ID, COLUMNS.BC, COLUMNS.METASUB_NAME, COLUMNS.SL_NAME]
)

# Athena
ATHENA = Namespace()
ATHENA.METASUB_RESULTS = '/athena/masonlab/scratch/projects/metagenomics/metasub/analysis/new_metasub_cap/.module_ultra/core_results'
ATHENA.METASUB_DATA = '/athena/masonlab/scratch/projects/metagenomics/metasub/data'
ATHENA.HALPHA_LIBRARY = '/athena/masonlab/scratch/projects/metagenomics/metasub/data/hudson_alpha_library'

# Hudson Alpha
HALPHA = Namespace()
HALPHA.URL = 'http://gsldl.hudsonalpha.org/'
HALPHA.FLOWCELL_FILENAME = join(CODE_DIR, 'tables/hudson_alpha_flowcells.csv')
HALPHA.ATHENA_SL_LIBRARY = join(ATHENA.METASUB_DATA, 'hudson_alpha_library')

# SFTP Server
ZURICH = Namespace()
ZURICH.URL = 'metasub.ethz.ch'
ZURICH.DATA = '/data/'
ZURICH.ASSEMBLIES = '/data/assemblies/'

# BRIDGES
BRIDGES = Namespace()
BRIDGES.METASUB_DATA = '/home/dcdanko/pylon5/MetaSUB'
BRIDGES.ASSEMBLIES = '/home/dcdanko/pylon5/metasub_assemblies'

# Tables
TABLES = Namespace()
TABLES.SL_TABLE = join(CODE_DIR, 'tables/datasuper_to_hauniq.tsv')

# Wasabi
WASABI = Namespace()
WASABI.BUCKET_NAME = 'metasub'
WASABI.ENDPOINT_URL = 'https://s3.wasabisys.com'



import pysftp
from .constants import ZURICH


class SFTPKnex:

    def __init__(self, username, password, dryrun=False):
        self.dryrun = dryrun
        cnopts = pysftp.CnOpts()
        cnopts.hostkeys = None
        self.knex = pysftp.Connection(ZURICH.URL, username=username, password=password, cnopts=cnopts)

    def close(self):
        self.knex.close()

    def __enter__(self):
        pass

    def __exit__(self):
        return self.close()

    def upload_file(self, localpath, remotepath):
        print(f'[SFTP Knex] UPLOAD {localpath} {remotepath}')
        if self.dryrun:
            return
        self.knex.put(localpath, remotepath=remotepath)

    def make_symlink(self, source_path, target_path):
        print(f'[SFTP Knex] SYMLINK {source_path} {target_path}')
        if self.dryrun:
            return
        self.knex.symlink(source_path, target_path)

    def make_dirs(self, remote_dirs):
        print(f'[SFTP Knex] MAKEDIRS {remote_dirs}')
        if self.dryrun:
            return
        self.knex.makedirs(remote_dirs)



import pandas as pd
from .constants import METADATA, TABLES


def get_complete_metadata(uploadable=False):
    """Return the complete metadata file as a pandas dataframe."""
    if uploadable:
        return pd.read_csv(METADATA.UPLOADABLE_TABLE_URL, dtype=str, index_col=0)
    return pd.read_csv(METADATA.COMPLETE_TABLE_URL, dtype=str)


def get_canonical_city_names(lower=False):
    """Return a set of canonical city names."""
    city_tbl = pd.read_csv(METADATA.CANONICAL_CITIES_URL, dtype=str)
    city_names = set(city_tbl.ix[:, 0])
    if lower:
        city_names = {city_name.lower() for city_name in city_names}
    return city_names


def as_display_name(name):
    """ Return a name with spaces and first letters capitalized."""
    return ' '.join([
        tkn[:1].upper() + tkn[1:]
        for tkn in name.split('_')
    ])


def parse_sl_table():
    tbl = {}
    with open(TABLES.SL_TABLE) as sl_file:
        for line in sl_file:
            line = line.strip()
            if len(line) == 0:
                continue
            tkns = line.split()
            tbl[tkns[0]] = tkns[1]
    return tbl



import boto3
import botocore
from os.path import join, dirname, basename, isfile
from os import makedirs
from .constants import WASABI, ATHENA, BRIDGES
from glob import glob
from concurrent.futures import ThreadPoolExecutor
from sys import stderr


class WasabiBucket:
    """Represents the metasub data bucket on Wasabi (an s3 clone)."""

    def __init__(self, profile_name=None, threads=1):
        self.session = boto3.Session(profile_name=profile_name)
        self.s3 = self.session.resource('s3', endpoint_url=WASABI.ENDPOINT_URL)
        self.bucket = self.s3.Bucket(WASABI.BUCKET_NAME)
        self.executor = ThreadPoolExecutor(max_workers=threads)
        self.futures = [None] * 2 * threads
        self.ind = 0
        self.closed = False

    def add_job(self, job):
        assert not self.closed
        if self.futures[self.ind] is not None:
            self.futures[self.ind].result()
        future = self.executor.submit(job)
        self.futures[self.ind] = future
        self.ind = (self.ind + 1) % len(self.futures)

    def close(self):
        self.closed = True
        for future in self.futures:
            if future:
                future.result()

    def list_files(self):
        """Return a list of all files in the bucket."""
        return {key.key for key in self.bucket.objects.all()}

    def upload(self, local_file, remote_key, dryrun):
        print(f'WASABI UPLOADING {local_file} {remote_key}')
        if not dryrun:
            self.add_job(lambda: self.bucket.upload_file(local_file, remote_key))

    def download(self, key, local_path, dryrun):
        if type(key) is not str:
            key = key.key
        print(f'WASABI DOWNLOADING {key} {local_path}')
        if not dryrun:
            makedirs(dirname(local_path), exist_ok=True)
            self.add_job(lambda: self.bucket.download_file(key, local_path))

    def list_unassembled_data(self):
        all_assembled_keys = {
            basename(dirname(key)).split('.')[0]
            for key in self.list_files() if 'assemblies' in key
        }
        unassembled_data = {
            key for key in self.list_files()
            if 'data' == key.split('/')[0] and
            '_'.join(basename(key).split('_')[:3]) not in all_assembled_keys
        }
        return unassembled_data

    def download_unassembled_data(self, target_dir='data', dryrun=True):
        """Download data without contigs."""
        for key in self.list_unassembled_data():
            local_path = target_dir + '/' + key.split('data/')[1]
            if isfile(local_path):
                continue
            self.download(key, local_path, dryrun)

    def download_contigs(self,
                         target_dir='assemblies', contig_file='contigs.fasta', dryrun=True):
        """Download contigs."""
        for key in self.bucket.objects.all():
            if 'assemblies' not in key.key or contig_file != basename(key.key):
                continue

            key_path = key.key.split('assemblies/')[1]
            key_dirs = dirname(key_path)
            local_path = join(
                target_dir,
                key_dirs,
                contig_file,
            )
            if isfile(local_path):
                continue
            self.download(key, local_path, dryrun)

    def upload_raw_data(self, data_dir=ATHENA.HALPHA_LIBRARY, dryrun=True):
        all_uploaded_results = {
            basename(key)
            for key in self.list_files()
            if 'data' in key
        }
        for result_file in glob(f'{data_dir}/*/*/*'):
            if basename(result_file) in all_uploaded_results:
                continue
            tkns = result_file.split('/')
            remote_key = f'data/{data_dir}/{tkns[-3]}/{tkns[-2]}/{tkns[-1]}'
            self.upload(result_file, remote_key, dryrun)

    def upload_results(self, result_dir=ATHENA.METASUB_RESULTS, dryrun=True):
        all_uploaded_results = {
            basename(key)
            for key in self.list_files()
            if 'cap_analysis' in key
        }
        for result_file in glob(f'{result_dir}/*/*'):
            if basename(result_file) in all_uploaded_results:
                continue
            tkns = result_file.split('/')
            remote_key = f'cap_analysis/{tkns[-2]}/{tkns[-1]}'
            try:
                self.upload(result_file, remote_key, dryrun)
            except:
                print(f'ERROR {result_file}', file=stderr)

    def upload_contigs(self, result_dir=BRIDGES.ASSEMBLIES, dryrun=True):
        all_uploaded_results = {
            basename(dirname(key))
            for key in self.list_files()
            if 'assemblies' in key
        }
        for result_file in glob(f'{result_dir}/**/*'):
            if basename(dirname(result_file)) in all_uploaded_results:
                continue
            result_root = result_file.split(result_dir)[1]
            remote_key = f'assemblies/{result_root}'
            self.upload(result_file, remote_key, dryrun)



# -*- coding: utf-8 -*-

"""Console script for metasub_utils."""

import click
from sys import (
    stderr,
    exit,
)
from .constants import ATHENA, BRIDGES
from .metagenscope import upload_cities
from .utils import (
    get_complete_metadata,
    get_canonical_city_names,
    as_display_name,
)
from .hudson_alpha import (
    process_flowcells,
    rename_sl_names_to_ha_unique,
    map_from_name_in_datasuper_to_ha_unique,
    rename_existing_core_results,
)
from .assemblies import (
    upload_metaspades_assemblies_from_bridges,
    copy_metaspades_assemblies_from_bridges,
)
from .wasabi_bucket import WasabiBucket


@click.group()
def main(args=None):
    """Console script for metasub_utils."""
    pass


###############################################################################


@main.group()
def get():
    pass


@get.command('cities')
def cli_get_canonical_city_names():
    """Print a list of canonical city names."""
    for city_name in get_canonical_city_names():
        print(city_name)


@get.command('metadata')
@click.option('--uploadable/--complete', default=False, help='optimize table for metagenscope')
def cli_get_metadata(uploadable):
    """Print a CSV with MetaSUB metadata."""
    tbl = get_complete_metadata(uploadable=uploadable)
    print(tbl.to_csv())


###############################################################################


@main.group()
def wasabi():
    pass


@wasabi.command('list')
@click.argument('profile_name', default='wasabi')
def cli_list_wasabi_files(profile_name):
    """List all files in the wasabi bucket."""
    wasabi_bucket = WasabiBucket(profile_name=profile_name)
    for file_key in wasabi_bucket.list_files():
        print(file_key)


@wasabi.command('list-unassembled')
@click.argument('profile_name', default='wasabi')
def cli_list_unassembled_data(profile_name):
    """List all files in the wasabi bucket."""
    wasabi_bucket = WasabiBucket(profile_name=profile_name)
    for file_key in wasabi_bucket.list_unassembled_data():
        print(file_key)


@wasabi.command('download-unassembled-data')
@click.option('-d/-w', '--dryrun/--wetrun', default=True)
@click.option('-p', '--profile-name', default='wasabi')
@click.argument('target_dir', default='data')
def cli_download_contig_files(dryrun, profile_name, target_dir):
    """Download data without contig files from wasabi."""
    wasabi_bucket = WasabiBucket(profile_name=profile_name)
    wasabi_bucket.download_unassembled_data(
        target_dir=target_dir,
        dryrun=dryrun,
    )
    wasabi_bucket.close()


@wasabi.command('download-contigs')
@click.option('-d/-w', '--dryrun/--wetrun', default=True)
@click.option('-p', '--profile-name', default='wasabi')
@click.argument('target_dir', default='assemblies')
def cli_download_contig_files(dryrun, profile_name, target_dir):
    """Download contig files from wasabi."""
    wasabi_bucket = WasabiBucket(profile_name=profile_name)
    wasabi_bucket.download_contigs(
        target_dir=target_dir,
        dryrun=dryrun,
    )
    wasabi_bucket.close()


@wasabi.command('upload-results')
@click.option('-d/-w', '--dryrun/--wetrun', default=True)
@click.option('-p', '--profile-name', default='wasabi')
@click.argument('result_dir', default=ATHENA.METASUB_RESULTS)
def cli_upload_results(dryrun, profile_name, result_dir):
    """Upload CAP results to wasabi. Only works on Athena."""
    wasabi_bucket = WasabiBucket(profile_name=profile_name)
    wasabi_bucket.upload_results(
        result_dir=result_dir,
        dryrun=dryrun,
    )
    wasabi_bucket.close()

@wasabi.command('upload-data')
@click.option('-d/-w', '--dryrun/--wetrun', default=True)
@click.option('-p', '--profile-name', default='wasabi')
@click.argument('data_dir', default=ATHENA.HALPHA_LIBRARY)
def cli_upload_raw_data(dryrun, profile_name, data_dir):
    """Upload CAP results to wasabi. Only works on Athena."""
    wasabi_bucket = WasabiBucket(profile_name=profile_name)
    wasabi_bucket.upload_raw_data(
        data_dir=data_dir,
        dryrun=dryrun,
    )
    wasabi_bucket.close()

@wasabi.command('upload-contigs')
@click.option('-d/-w', '--dryrun/--wetrun', default=True)
@click.option('-p', '--profile-name', default='wasabi')
@click.argument('result_dir', default=BRIDGES.ASSEMBLIES)
def cli_upload_contigs(dryrun, profile_name, result_dir):
    """Upload CAP results to wasabi. Only works on bridges."""
    wasabi_bucket = WasabiBucket(profile_name=profile_name)
    wasabi_bucket.upload_contigs(
        result_dir=result_dir,
        dryrun=dryrun,
    )
    wasabi_bucket.close()


###############################################################################


@main.group()
def upload():
    pass


@upload.command(name='city')
@click.option('--dryrun/--wetrun', default=True)
@click.option('--upload-only/--run-middleware', default=False)
@click.option('-n', '--display-name', default=None)
@click.option('--display-name-suffix', default=None)
@click.argument('city_names', nargs=-1)
def cli_upload_city(dryrun, upload_only, display_name, display_name_suffix, city_names):
    """Upload a city to MetaGenScope. Only works on Athena."""
    result_dir = ATHENA.METASUB_RESULTS
    if len(city_names) == 1 and display_name is None:
        display_name = as_display_name(city_names[0])
    elif len(city_names) > 1 and display_name is None:
        print('Display name cannot be blank if multiple cities are listed', file=stderr)
        exit(1)
    if display_name_suffix:
        display_name += display_name_suffix
    upload_cities(
        result_dir, city_names, display_name,
        upload_only=upload_only,
        dryrun=dryrun
    )


@upload.command(name='assemblies')
@click.option('--dryrun/--wetrun', default=True)
@click.argument('username')
@click.argument('password')
def cli_upload_assemblies(dryrun, username, password):
    """Upload assemblies from bridges to Zurich SFTP"""
    upload_metaspades_assemblies_from_bridges(username, password, dryrun=dryrun)


###############################################################################


@main.group()
def copy():
    pass


@copy.command(name='assemblies')
@click.argument('target_dir')
def cli_copy_assemblies(target_dir):
    """Copy assemblies to a new dir on bridges."""
    copy_metaspades_assemblies_from_bridges(target_dir)

###############################################################################


@main.group()
def download():
    pass


@download.command(name='ha')
@click.option('--dryrun/--wetrun', default=True)
@click.argument('username')
@click.argument('password')
def cli_download_ha_files(dryrun, username, password):
    """Download flowcells from Hudson Alpha, intended for Athena."""
    process_flowcells(dryrun, (username, password))


###############################################################################


@main.group()
def athena():
    pass


@athena.command(name='rename-sl-files')
def cli_rename_sl_files():
    """Print a two column file of old path to new path."""
    rename_sl_names_to_ha_unique()


@athena.command(name='datasuper-to-hauniq')
@click.argument('source_fastqs', type=click.File('r'))
def cli_datasuper_to_hauniq(source_fastqs):
    """Print a two column file of datasuper name to ha uniq."""
    for ds_name, ha_uniq in map_from_name_in_datasuper_to_ha_unique(source_fastqs).items():
        print(f'{ds_name} {ha_uniq}')


@athena.command(name='new-core-results')
@click.argument('source_fastqs', type=click.File('r'))
@click.argument('old_result_dir')
def cli_new_core_results(source_fastqs, old_result_dir):
    """Print a two column file of old result path to result path with ha uniq."""
    for ds_name, ha_uniq in rename_existing_core_results(source_fastqs, old_result_dir).items():
        print(f'{ds_name} {ha_uniq}')


###############################################################################


if __name__ == "__main__":
    main()
